@@ -3,7 +3,7 @@
    Protocol structures... */
 
 /*
- * Copyright (c) 2004-2005 by Internet Systems Consortium, Inc. ("ISC")
+ * Copyright (c) 2004-2007 by Internet Systems Consortium, Inc. ("ISC")
  * Copyright (c) 1995-2003 by Internet Software Consortium
  *
  * Permission to use, copy, modify, and distribute this software for any
@@ -30,18 +30,23 @@
  * To learn more about Vixie Enterprises, see ``http://www.vix.com''.
  */
 
+#ifndef DHCP_H
+#define DHCP_H
+
 #define DHCP_UDP_OVERHEAD	(20 + /* IP header */			\
-			8)   /* UDP header */
+			        8)   /* UDP header */
 #define DHCP_SNAME_LEN		64
 #define DHCP_FILE_LEN		128
 #define DHCP_FIXED_NON_UDP	236
 #define DHCP_FIXED_LEN		(DHCP_FIXED_NON_UDP + DHCP_UDP_OVERHEAD)
 						/* Everything but options. */
-#define DHCP_MTU_MAX		1500
-#define DHCP_OPTION_LEN		(DHCP_MTU_MAX - DHCP_FIXED_LEN)
-
 #define BOOTP_MIN_LEN		300
-#define DHCP_MIN_LEN            548
+
+#define DHCP_MTU_MAX		1500
+#define DHCP_MTU_MIN            576
+
+#define DHCP_MAX_OPTION_LEN	(DHCP_MTU_MAX - DHCP_FIXED_LEN)
+#define DHCP_MIN_OPTION_LEN     (DHCP_MTU_MIN - DHCP_FIXED_LEN)
 
 struct dhcp_packet {
  u_int8_t  op;		/* 0: Message opcode/type */
@@ -58,7 +63,7 @@ struct dhcp_packet {
 	unsigned char chaddr [16];	/* 24: Client hardware address */
 	char sname [DHCP_SNAME_LEN];	/* 40: Server name */
 	char file [DHCP_FILE_LEN];	/* 104: Boot filename */
-	unsigned char options [DHCP_OPTION_LEN];
+	unsigned char options [DHCP_MAX_OPTION_LEN];
 				/* 212: Optional parameters
 			  (actual length dependent on MTU). */
 };
@@ -197,3 +202,6 @@ struct dhcp_packet {
 
 /* Enterprise Suboptions: */
 #define VENDOR_ISC_SUBOPTIONS		2495
+
+#endif /* DHCP_H */
+
