@@ -52,7 +52,7 @@
  */
 
 /*
- * Portions Copyright (c) 2004 by Internet Systems Consortium, Inc. ("ISC")
+ * Portions Copyright (c) 2004,2007 by Internet Systems Consortium, Inc. ("ISC")
  * Portions Copyright (c) 1996-2003 by Internet Software Consortium
  *
  * Permission to use, copy, modify, and distribute this software for any
@@ -71,12 +71,12 @@
  *   950 Charter Street
  *   Redwood City, CA 94063
  *   <info@isc.org>
- *   https://www.isc.org/
+ *   http://www.isc.org/
  */
 
 #if defined(LIBC_SCCS) && !defined(lint)
 static const char sccsid[] = "@(#)res_query.c	8.1 (Berkeley) 6/4/93";
-static const char rcsid[] = "$Id: res_query.c,v 1.5.140.2 2009/07/23 21:43:35 sar Exp $";
+static const char rcsid[] = "$Id: res_query.c,v 1.8 2007/09/05 17:32:10 dhankins Exp $";
 #endif /* LIBC_SCCS and not lint */
 
 #include <sys/types.h>
@@ -396,7 +396,7 @@ res_hostalias(const res_state statp, const char *name, char *dst, size_t siz) {
 	setbuf(fp, NULL);
 	buf[sizeof(buf) - 1] = '\0';
 	while (fgets(buf, sizeof(buf), fp)) {
-		for (cp1 = buf; *cp1 && !isspace(*cp1); ++cp1)
+		for (cp1 = (unsigned char *)buf; *cp1 && !isspace(*cp1); ++cp1)
 			;
 		if (!*cp1)
 			break;
@@ -409,7 +409,7 @@ res_hostalias(const res_state statp, const char *name, char *dst, size_t siz) {
 			for (cp2 = cp1 + 1; *cp2 && !isspace(*cp2); ++cp2)
 				;
 			*cp2 = '\0';
-			strncpy(dst, cp1, siz - 1);
+			strncpy(dst, (char *)cp1, siz - 1);
 			dst[siz - 1] = '\0';
 			fclose(fp);
 			return (dst);
