@@ -3,7 +3,7 @@
    Subroutines for dealing with connections. */
 
 /*
- * Copyright (c) 2004 by Internet Systems Consortium, Inc. ("ISC")
+ * Copyright (c) 2004,2007 by Internet Systems Consortium, Inc. ("ISC")
  * Copyright (c) 1999-2003 by Internet Software Consortium
  *
  * Permission to use, copy, modify, and distribute this software for any
@@ -22,19 +22,22 @@
  *   950 Charter Street
  *   Redwood City, CA 94063
  *   <info@isc.org>
- *   https://www.isc.org/
+ *   http://www.isc.org/
  *
  * This software has been written for Internet Systems Consortium
  * by Ted Lemon in cooperation with Vixie Enterprises and Nominum, Inc.
  * To learn more about Internet Systems Consortium, see
- * ``https://www.isc.org/''.  To learn more about Vixie Enterprises,
+ * ``http://www.isc.org/''.  To learn more about Vixie Enterprises,
  * see ``http://www.vix.com''.   To learn more about Nominum, Inc., see
  * ``http://www.nominum.com''.
  */
 
+#include "dhcpd.h"
+
 #include <omapip/omapip_p.h>
 #include <arpa/inet.h>
 #include <arpa/nameser.h>
+#include <errno.h>
 
 
 #if defined (TRACING)
@@ -111,10 +114,6 @@ isc_result_t omapi_connect_list (omapi_object_t *c,
 	omapi_connection_object_t *obj;
 	int flag;
 	struct sockaddr_in local_sin;
-#if defined (TRACING)
-	trace_addr_t *addrs;
-	u_int16_t naddrs;
-#endif
 
 	obj = (omapi_connection_object_t *)0;
 	status = omapi_connection_allocate (&obj, MDL);
@@ -174,8 +173,9 @@ isc_result_t omapi_connect_list (omapi_object_t *c,
 			
 			if (bind (obj -> socket, (struct sockaddr *)&local_sin,
 				  sizeof local_sin) < 0) {
-				omapi_object_dereference ((omapi_object_t **)
-							  &obj, MDL);
+				omapi_connection_object_t **objp = &obj;
+				omapi_object_t **o = (omapi_object_t **)objp;
+				omapi_object_dereference(o, MDL);
 				if (errno == EADDRINUSE)
 					return ISC_R_ADDRINUSE;
 				if (errno == EADDRNOTAVAIL)
@@ -187,7 +187,7 @@ isc_result_t omapi_connect_list (omapi_object_t *c,
 			obj -> local_addr = local_sin;
 		}
 
-#if defined (HAVE_SETFD)
+#if defined(F_SETFD)
 		if (fcntl (obj -> socket, F_SETFD, 1) < 0) {
 			close (obj -> socket);
 			omapi_connection_dereference (&obj, MDL);
@@ -383,9 +383,6 @@ static void trace_connect_input (trace_type_t *ttype,
 			lp -> state = omapi_connection_connected;
 			lp -> remote_addr = remote;
 			lp -> remote_addr.sin_family = AF_INET;
-#if defined (HAVE_SIN_LEN)
-			lp -> remote_addr.sin_len = sizeof remote;
-#endif
 			omapi_addr_list_dereference (&lp -> connect_list, MDL);
 			lp -> index = connect_index;
 			status = omapi_signal_in ((omapi_object_t *)lp,
@@ -590,7 +587,7 @@ static isc_result_t omapi_connection_connect_internal (omapi_object_t *h)
 {
 	int error;
 	omapi_connection_object_t *c;
-	SOCKLEN_T sl;
+	socklen_t sl;
 	isc_result_t status;
 
 	if (h -> type != omapi_type_connection)
@@ -1027,8 +1024,6 @@ isc_result_t omapi_connection_stuff_values (omapi_object_t *c,
 					    omapi_object_t *id,
 					    omapi_object_t *m)
 {
-	int i;
-
 	if (m -> type != omapi_type_connection)
 		return ISC_R_INVALIDARG;
 
