@@ -3,7 +3,7 @@
    BOOTP Protocol support. */
 
 /*
- * Copyright (c) 2004-2006,2008 by Internet Systems Consortium, Inc. ("ISC")
+ * Copyright (c) 2004-2008 by Internet Systems Consortium, Inc. ("ISC")
  * Copyright (c) 1995-2003 by Internet Software Consortium
  *
  * Permission to use, copy, modify, and distribute this software for any
@@ -22,22 +22,18 @@
  *   950 Charter Street
  *   Redwood City, CA 94063
  *   <info@isc.org>
- *   https://www.isc.org/
+ *   http://www.isc.org/
  *
  * This software has been written for Internet Systems Consortium
  * by Ted Lemon in cooperation with Vixie Enterprises and Nominum, Inc.
  * To learn more about Internet Systems Consortium, see
- * ``https://www.isc.org/''.  To learn more about Vixie Enterprises,
+ * ``http://www.isc.org/''.  To learn more about Vixie Enterprises,
  * see ``http://www.vix.com''.   To learn more about Nominum, Inc., see
  * ``http://www.nominum.com''.
  */
 
-#ifndef lint
-static char copyright[] =
-"$Id: bootp.c,v 1.76.2.4 2009/07/23 21:43:35 sar Exp $ Copyright (c) 2004-2006,2008 Internet Systems Consortium.  All rights reserved.\n";
-#endif /* not lint */
-
 #include "dhcpd.h"
+#include <errno.h>
 
 #if defined (TRACING)
 # define send_packet trace_packet_send
@@ -300,11 +296,12 @@ void bootp (packet)
 			memcpy (&raw.siaddr, d1.data, 4);
 		data_string_forget (&d1, MDL);
 	} else {
-		if (lease -> subnet -> shared_network -> interface)
-			raw.siaddr = (lease -> subnet -> shared_network ->
-				      interface -> primary_address);
-		else
-			raw.siaddr = packet -> interface -> primary_address;
+		if ((lease->subnet->shared_network->interface != NULL) &&
+		    lease->subnet->shared_network->interface->address_count)
+		    raw.siaddr =
+			lease->subnet->shared_network->interface->addresses[0];
+		else if (packet->interface->address_count)
+			raw.siaddr = packet->interface->addresses[0];
 	}
 
 	raw.giaddr = packet -> raw -> giaddr;
@@ -354,7 +351,14 @@ void bootp (packet)
 	hto.hlen = packet -> raw -> hlen + 1;
 	memcpy (&hto.hbuf [1], packet -> raw -> chaddr, packet -> raw -> hlen);
 
-	from = packet -> interface -> primary_address;
+	if (packet->interface->address_count) {
+		from = packet->interface->addresses[0];
+	} else {
+		log_error("%s: Interface %s appears to have no IPv4 "
+			  "addresses, and so dhcpd cannot select a source "
+			  "address.", msgbuf, packet->interface->name);
+		goto out;
+	}
 
 	/* Report what we're doing... */
 	log_info ("%s", msgbuf);
