@@ -3,7 +3,7 @@
    Server-specific in-memory database support. */
 
 /*
- * Copyright (c) 2004-2009 by Internet Systems Consortium, Inc. ("ISC")
+ * Copyright (c) 2004-2008 by Internet Systems Consortium, Inc. ("ISC")
  * Copyright (c) 1996-2003 by Internet Software Consortium
  *
  * Permission to use, copy, modify, and distribute this software for any
@@ -22,21 +22,16 @@
  *   950 Charter Street
  *   Redwood City, CA 94063
  *   <info@isc.org>
- *   https://www.isc.org/
+ *   http://www.isc.org/
  *
  * This software has been written for Internet Systems Consortium
  * by Ted Lemon in cooperation with Vixie Enterprises and Nominum, Inc.
  * To learn more about Internet Systems Consortium, see
- * ``https://www.isc.org/''.  To learn more about Vixie Enterprises,
+ * ``http://www.isc.org/''.  To learn more about Vixie Enterprises,
  * see ``http://www.vix.com''.   To learn more about Nominum, Inc., see
  * ``http://www.nominum.com''.
  */
 
-#ifndef lint
-static char copyright[] =
-"$Id: mdb.c,v 1.83.16.15 2009/09/01 20:32:28 dhankins Exp $ Copyright (c) 2004-2008 Internet Systems Consortium.  All rights reserved.\n";
-#endif /* not lint */
-
 #include "dhcpd.h"
 #include "omapip/hash.h"
 
@@ -49,6 +44,25 @@ lease_id_hash_t *lease_uid_hash;
 lease_ip_hash_t *lease_ip_addr_hash;
 lease_id_hash_t *lease_hw_addr_hash;
 
+/*
+ * We allow users to specify any option as a host identifier.
+ *
+ * Any host is uniquely identified by the combination of 
+ * option type & option data.
+ *
+ * We expect people will only use a few types of options as host 
+ * identifier. Because of this, we store a list with an entry for
+ * each option type. Each of these has a hash table, which contains 
+ * hash of the option data.
+ */
+typedef struct host_id_info {
+	struct option *option;
+	host_hash_t *values_hash;
+	struct host_id_info *next;
+} host_id_info_t;
+
+static host_id_info_t *host_id_info = NULL;
+
 int numclasseswritten;
 
 omapi_object_type_t *dhcp_type_host;
@@ -129,6 +143,77 @@ static int find_uid_statement (struct executable_statement *esp,
 	return 0;
 }
 
+
+static host_id_info_t *
+find_host_id_info(unsigned int option_code) {
+	host_id_info_t *p;
+
+	for (p=host_id_info; p != NULL; p = p->next) {
+		if (p->option->code == option_code) {
+			break;
+		}
+	}
+	return p;
+}
+
+/* Debugging code */
+#if 0
+isc_result_t
+print_host(const void *name, unsigned len, void *value) {
+	struct host_decl *h;
+	printf("--------------\n");
+	printf("name:'%s'\n", print_hex_1(len, name, 60));
+	printf("len:%d\n", len);
+	h = (struct host_decl *)value;
+	printf("host @%p is '%s'\n", h, h->name);
+	return ISC_R_SUCCESS;
+}
+
+void
+hash_print_hosts(struct hash_table *h) {
+	hash_foreach(h, print_host);
+	printf("--------------\n");
+}
+#endif /* 0 */
+
+void
+change_host_uid(struct host_decl *host, const char *uid, int len) {
+	/* XXX: should consolidate this type of code throughout */
+	if (host_uid_hash == NULL) {
+		if (!host_new_hash(&host_uid_hash, HOST_HASH_SIZE, MDL)) {
+			log_fatal("Can't allocate host/uid hash");
+		}
+	}
+
+	/* 
+	 * Remove the old entry, if one exists.
+	 */
+	if (host->client_identifier.data != NULL) {
+		host_hash_delete(host_uid_hash,
+				 host->client_identifier.data,
+				 host->client_identifier.len,
+				 MDL);
+		data_string_forget(&host->client_identifier, MDL);
+	}
+
+	/* 
+	 * Set our new value.
+	 */
+	memset(&host->client_identifier, 0, sizeof(host->client_identifier));
+	host->client_identifier.len = len;
+	if (!buffer_allocate(&host->client_identifier.buffer, len, MDL)) {
+		log_fatal("Can't allocate uid buffer");
+	}
+	host->client_identifier.data = host->client_identifier.buffer->data;
+	memcpy((char *)host->client_identifier.data, uid, len);
+
+	/*
+	 * And add to hash.
+	 */
+	host_hash_add(host_uid_hash, host->client_identifier.data, 
+		      host->client_identifier.len, host, MDL);
+}
+
 isc_result_t enter_host (hd, dynamicp, commit)
 	struct host_decl *hd;
 	int dynamicp;
@@ -137,6 +222,7 @@ isc_result_t enter_host (hd, dynamicp, commit)
 	struct host_decl *hp = (struct host_decl *)0;
 	struct host_decl *np = (struct host_decl *)0;
 	struct executable_statement *esp;
+	host_id_info_t *h_id_info;
 
 	if (!host_name_hash) {
 		if (!host_new_hash(&host_name_hash, HOST_HASH_SIZE, MDL))
@@ -276,6 +362,63 @@ isc_result_t enter_host (hd, dynamicp, commit)
 		}
 	}
 
+
+	/*
+	 * If we use an option as our host identifier, record it here.
+	 */
+	if (hd->host_id_option != NULL) {
+		/*
+		 * Look for the host identifier information for this option,
+		 * and create a new entry if there is none.
+		 */
+		h_id_info = find_host_id_info(hd->host_id_option->code);
+		if (h_id_info == NULL) {
+			h_id_info = dmalloc(sizeof(*h_id_info), MDL);
+			if (h_id_info == NULL) {
+				log_fatal("No memory for host-identifier "
+					  "option information.");
+			}
+			option_reference(&h_id_info->option, 
+					 hd->host_id_option, MDL);
+			if (!host_new_hash(&h_id_info->values_hash, 
+					   HOST_HASH_SIZE, MDL)) {
+				log_fatal("No memory for host-identifier "
+					  "option hash.");
+			}
+			h_id_info->next = host_id_info;
+			host_id_info = h_id_info;
+		}
+
+		if (host_hash_lookup(&hp, h_id_info->values_hash, 
+				     hd->host_id.data, hd->host_id.len, MDL)) {
+			/* 
+			 * If this option is already present, then add 
+			 * this host to the list in n_ipaddr, unless
+			 * we have already done so previously.
+			 *
+			 * XXXSK: This seems scary to me, but I don't
+			 *        fully understand how these are used. 
+			 *        Shouldn't there be multiple lists, or 
+			 *        maybe we should just forbid duplicates?
+			 */
+			if (np == NULL) {
+				np = hp;
+				while (np->n_ipaddr != NULL) {
+					np = np->n_ipaddr;
+				}
+				if (hd != np) {
+					host_reference(&np->n_ipaddr, hd, MDL);
+				}
+			}
+			host_dereference(&hp, MDL);
+		} else {
+			host_hash_add(h_id_info->values_hash, 
+				      hd->host_id.data,
+				      hd->host_id.len,
+				      hd, MDL);
+		}
+	}
+
 	if (dynamicp && commit) {
 		if (!write_host (hd))
 			return ISC_R_IOERROR;
@@ -317,7 +460,6 @@ isc_result_t delete_host (hd, commit)
 	struct host_decl *hp = (struct host_decl *)0;
 	struct host_decl *np = (struct host_decl *)0;
 	struct host_decl *foo;
-	struct executable_statement *esp;
 	int hw_head = 0, uid_head = 1;
 
 	/* Don't need to do it twice. */
@@ -409,6 +551,11 @@ isc_result_t delete_host (hd, commit)
 	    }
 	}
 
+	if (hd->host_id_option != NULL) {
+		option_dereference(&hd->host_id_option, MDL);
+		data_string_forget(&hd->host_id, MDL);
+	}
+
 	if (hd -> n_ipaddr) {
 		if (uid_head && hd -> n_ipaddr -> client_identifier.len) {
 			host_hash_add
@@ -452,7 +599,6 @@ int find_hosts_by_haddr (struct host_decl **hp, int htype,
 			 const unsigned char *haddr, unsigned hlen,
 			 const char *file, int line)
 {
-	struct host_decl *foo;
 	struct hardware h;
 
 	h.hlen = hlen + 1;
@@ -470,6 +616,43 @@ int find_hosts_by_uid (struct host_decl **hp,
 	return host_hash_lookup (hp, host_uid_hash, data, len, file, line);
 }
 
+int
+find_hosts_by_option(struct host_decl **hp, 
+		     struct packet *packet,
+		     struct option_state *opt_state,
+		     const char *file, int line) {
+	host_id_info_t *p;
+	struct option_cache *oc;
+	struct data_string data;
+	int found;
+	
+	for (p = host_id_info; p != NULL; p = p->next) {
+		oc = lookup_option(p->option->universe, 
+				   opt_state, p->option->code);
+		if (oc != NULL) {
+			memset(&data, 0, sizeof(data));
+			if (!evaluate_option_cache(&data, packet, NULL, NULL,
+						   opt_state, NULL,
+						   &global_scope, oc, 
+						   MDL)) {
+				log_error("Error evaluating option cache");
+				return 0;
+			}
+			
+			found = host_hash_lookup(hp, p->values_hash, 
+						 data.data, data.len,
+						 file, line);
+
+			data_string_forget(&data, MDL);
+
+			if (found) {
+				return 1;
+			}
+		}
+	}
+	return 0;
+}
+
 /* More than one host_decl can be returned by find_hosts_by_haddr or
    find_hosts_by_uid, and each host_decl can have multiple addresses.
    Loop through the list of hosts, and then for each host, through the
@@ -482,7 +665,6 @@ int find_host_for_network (struct subnet **sp, struct host_decl **host,
 			   struct iaddr *addr, struct shared_network *share)
 {
 	int i;
-	struct subnet *subnet;
 	struct iaddr ip_address;
 	struct host_decl *hp;
 	struct data_string fixed_addr;
@@ -532,15 +714,14 @@ void new_address_range (cfile, low, high, subnet, pool, lpchain)
 	struct pool *pool;
 	struct lease **lpchain;
 {
-#if defined(COMPACT_LEASES)
 	struct lease *address_range;
-#endif
-	struct lease *lp, *plp;
 	unsigned min, max, i;
 	char lowbuf [16], highbuf [16], netbuf [16];
 	struct shared_network *share = subnet -> shared_network;
-	isc_result_t status;
 	struct lease *lt = (struct lease *)0;
+#if !defined(COMPACT_LEASES)
+	isc_result_t status;
+#endif
 
 	/* All subnets should have attached shared network structures. */
 	if (!share) {
@@ -686,29 +867,29 @@ int find_grouped_subnet (struct subnet **sp,
 	return 0;
 }
 
-int subnet_inner_than (subnet, scan, warnp)
-	struct subnet *subnet, *scan;
-	int warnp;
-{
-	if (addr_eq (subnet_number (subnet -> net, scan -> netmask),
-		     scan -> net) ||
-	    addr_eq (subnet_number (scan -> net, subnet -> netmask),
-		     subnet -> net)) {
-		char n1buf [16];
+/* XXX: could speed up if everyone had a prefix length */
+int 
+subnet_inner_than(const struct subnet *subnet, 
+		  const struct subnet *scan,
+		  int warnp) {
+	if (addr_eq(subnet_number(subnet->net, scan->netmask), scan->net) ||
+	    addr_eq(subnet_number(scan->net, subnet->netmask), subnet->net)) {
+		char n1buf[sizeof("ffff:ffff:ffff:ffff:ffff:ffff:255.255.255")];
 		int i, j;
-		for (i = 0; i < 32; i++)
-			if (subnet -> netmask.iabuf [3 - (i >> 3)]
+		for (i = 0; i < 128; i++)
+			if (subnet->netmask.iabuf[3 - (i >> 3)]
 			    & (1 << (i & 7)))
 				break;
-		for (j = 0; j < 32; j++)
-			if (scan -> netmask.iabuf [3 - (j >> 3)] &
+		for (j = 0; j < 128; j++)
+			if (scan->netmask.iabuf[3 - (j >> 3)] &
 			    (1 << (j & 7)))
 				break;
-		strcpy (n1buf, piaddr (subnet -> net));
-		if (warnp)
-			log_error ("%ssubnet %s/%d overlaps subnet %s/%d",
-			      "Warning: ", n1buf, 32 - i,
-			      piaddr (scan -> net), 32 - j);
+		if (warnp) {
+			strcpy(n1buf, piaddr(subnet->net));
+			log_error("Warning: subnet %s/%d overlaps subnet %s/%d",
+			      n1buf, 32 - i,
+			      piaddr(scan->net), 32 - j);
+		}
 		if (i < j)
 			return 1;
 	}
@@ -824,7 +1005,6 @@ void enter_lease (lease)
 	struct lease *lease;
 {
 	struct lease *comp = (struct lease *)0;
-	isc_result_t status;
 
 	if (find_lease_by_ip_addr (&comp, lease -> ip_addr, MDL)) {
 		if (!comp -> pool) {
@@ -871,7 +1051,6 @@ int supersede_lease (comp, lease, commit, propogate, pimmediate)
 	int pimmediate;
 {
 	struct lease *lp, **lq, *prev;
-	TIME lp_next_state;
 #if defined (FAILOVER_PROTOCOL)
 	int do_pool_check = 0;
 
@@ -1040,7 +1219,7 @@ int supersede_lease (comp, lease, commit, propogate, pimmediate)
       just_move_it:
 #if defined (FAILOVER_PROTOCOL)
 	/* Atsfp should be cleared upon any state change that implies
-	 * propogation wether supersede_lease was given a copy lease
+	 * propagation whether supersede_lease was given a copy lease
 	 * structure or not (often from the pool_timer()).
 	 */
 	if (propogate)
@@ -1056,12 +1235,9 @@ int supersede_lease (comp, lease, commit, propogate, pimmediate)
 	/* Figure out which queue it's on. */
 	switch (comp -> binding_state) {
 	      case FTS_FREE:
-		if (comp->flags & RESERVED_LEASE)
-			lq = &comp->pool->reserved;
-		else {
-			lq = &comp->pool->free;
+		lq = &comp -> pool -> free;
+		if (!(comp->flags & RESERVED_LEASE))
 			comp->pool->free_leases--;
-		}
 
 #if defined(FAILOVER_PROTOCOL)
 		do_pool_check = 1;
@@ -1083,12 +1259,9 @@ int supersede_lease (comp, lease, commit, propogate, pimmediate)
 		break;
 
 	      case FTS_BACKUP:
-		if (comp->flags & RESERVED_LEASE)
-			lq = &comp->pool->reserved;
-		else {
-			lq = &comp->pool->backup;
+		lq = &comp -> pool -> backup;
+		if (!(comp->flags & RESERVED_LEASE))
 			comp->pool->backup_leases--;
-		}
 
 #if defined(FAILOVER_PROTOCOL)
 		do_pool_check = 1;
@@ -1227,22 +1400,7 @@ void make_binding_state_transition (struct lease *lease)
 	      lease -> binding_state == FTS_ACTIVE &&
 	      lease -> next_binding_state != FTS_RELEASED))) {
 #if defined (NSUPDATE)
-		/*
-		 * Note: ddns_removals() is also iterated when the lease
-		 * enters state 'released' in 'release_lease()'.  The below
-		 * is caught when a peer receives a BNDUPD from a failover
-		 * peer; it may not have received the client's release (it
-		 * may have been offline).
-		 *
-		 * We could remove the call from release_lease() because
-		 * it will also catch here on the originating server after the
-		 * peer acknowledges the state change.  However, there could
-		 * be many hours inbetween, and in this case we /know/ the
-		 * client is no longer using the lease when we receive the
-		 * release message.  This is not true of expiry, where the
-		 * peer may have extended the lease.
-		 */
-		ddns_removals (lease);
+		ddns_removals(lease, NULL);
 #endif
 		if (lease -> on_expiry) {
 			execute_statements ((struct binding_value **)0,
@@ -1293,7 +1451,7 @@ void make_binding_state_transition (struct lease *lease)
 	      lease -> binding_state == FTS_ACTIVE &&
 	      lease -> next_binding_state == FTS_RELEASED))) {
 #if defined (NSUPDATE)
-		ddns_removals (lease);
+		ddns_removals(lease, NULL);
 #endif
 		if (lease -> on_release) {
 			execute_statements ((struct binding_value **)0,
@@ -1460,7 +1618,7 @@ void release_lease (lease, packet)
 	/* If there are statements to execute when the lease is
 	   released, execute them. */
 #if defined (NSUPDATE)
-	ddns_removals (lease);
+	ddns_removals(lease, NULL);
 #endif
 	if (lease -> on_release) {
 		execute_statements ((struct binding_value **)0,
@@ -1521,7 +1679,7 @@ void abandon_lease (lease, message)
 {
 	struct lease *lt = (struct lease *)0;
 #if defined (NSUPDATE)
-	ddns_removals (lease);
+	ddns_removals(lease, NULL);
 #endif
 
 	if (!lease_copy (&lt, lease, MDL))
@@ -1553,7 +1711,7 @@ void dissociate_lease (lease)
 {
 	struct lease *lt = (struct lease *)0;
 #if defined (NSUPDATE)
-	ddns_removals (lease);
+	ddns_removals(lease, NULL);
 #endif
 
 	if (!lease_copy (&lt, lease, MDL))
@@ -1584,7 +1742,6 @@ void pool_timer (vpool)
 	void *vpool;
 {
 	struct pool *pool;
-	struct lease *lt = (struct lease *)0;
 	struct lease *next = (struct lease *)0;
 	struct lease *lease = (struct lease *)0;
 #define FREE_LEASES 0
@@ -2110,6 +2267,11 @@ int write_leases ()
 	    }
 	}
 	log_info ("Wrote %d leases to leases file.", num_written);
+#ifdef DHCPv6
+	if (!write_leases6()) {
+		return 0;
+	}
+#endif /* DHCPv6 */
 	if (!commit_leases ())
 		return 0;
 	return 1;
@@ -2206,7 +2368,7 @@ int lease_enqueue (struct lease *comp)
 	}
 
 	/* This only works during server startup: during runtime, the last
-	 * lease may be dequeued inbetween calls.  If the queue is the same
+	 * lease may be dequeued in between calls.  If the queue is the same
 	 * as was used previously, and the lease structure isn't (this is not
 	 * a re-queue), use that as a starting point for the insertion-sort.
 	 */
@@ -2308,7 +2470,6 @@ void expire_all_pools ()
 {
 	struct shared_network *s;
 	struct pool *p;
-	struct hash_bucket *hb;
 	int i;
 	struct lease *l;
 	struct lease **lptr[RESERVED_LEASES+1];
@@ -2439,7 +2600,7 @@ extern int end;
 extern struct lease *lease_hunks;
 #endif
 
-void free_everything(void)
+void free_everything ()
 {
 	struct subnet *sc = (struct subnet *)0, *sn = (struct subnet *)0;
 	struct shared_network *nc = (struct shared_network *)0,
@@ -2450,8 +2611,12 @@ void free_everything(void)
 		*in = (struct interface_info *)0;
 	struct class *cc = (struct class *)0, *cn = (struct class *)0;
 	struct collection *lp;
+	void *st = (shared_networks
+		    ? (shared_networks -> next
+		       ? shared_networks -> next -> next : 0) : 0);
 	int i;
 
+
 	/* Get rid of all the hash tables. */
 	if (host_hw_addr_hash)
 		host_free_hash_table (&host_hw_addr_hash, MDL);
@@ -2460,13 +2625,13 @@ void free_everything(void)
 		host_free_hash_table (&host_uid_hash, MDL);
 	host_uid_hash = 0;
 	if (lease_uid_hash)
-		lease_id_free_hash_table (&lease_uid_hash, MDL);
+		lease_free_hash_table (&lease_uid_hash, MDL);
 	lease_uid_hash = 0;
 	if (lease_ip_addr_hash)
-		lease_ip_free_hash_table (&lease_ip_addr_hash, MDL);
+		lease_free_hash_table (&lease_ip_addr_hash, MDL);
 	lease_ip_addr_hash = 0;
 	if (lease_hw_addr_hash)
-		lease_id_free_hash_table (&lease_hw_addr_hash, MDL);
+		lease_free_hash_table (&lease_hw_addr_hash, MDL);
 	lease_hw_addr_hash = 0;
 	if (host_name_hash)
 		host_free_hash_table (&host_name_hash, MDL);
@@ -2474,6 +2639,15 @@ void free_everything(void)
 	if (dns_zone_hash)
 		dns_zone_free_hash_table (&dns_zone_hash, MDL);
 	dns_zone_hash = 0;
+
+	while (host_id_info != NULL) {
+		host_id_info_t *tmp;
+		option_dereference(&host_id_info->option, MDL);
+		host_free_hash_table(&host_id_info->values_hash, MDL);
+		tmp = host_id_info->next;
+		dfree(host_id_info, MDL);
+		host_id_info = tmp;
+	}
 #if 0
 	if (auth_key_hash)
 		auth_key_free_hash_table (&auth_key_hash, MDL);
@@ -2563,15 +2737,7 @@ void free_everything(void)
 	}
 
 	/* So are shared networks. */
-	/* XXX: this doesn't work presently, but i'm ok just filtering
-	 * it out of the noise (you get a bigger spike on the real leaks).
-	 * It would be good to fix this, but it is not a "real bug," so not
-	 * today.  This hack is incomplete, it doesn't trim out sub-values.
-	 */
 	if (shared_networks) {
-		shared_network_dereference (&shared_networks, MDL);
-	/* This is the old method (tries to free memory twice, broken) */
-	} else if (0) {
 	    shared_network_reference (&nn, shared_networks, MDL);
 	    do {
 		if (nn) {
@@ -2667,21 +2833,14 @@ void free_everything(void)
 
 	universe_free_hash_table (&universe_hash, MDL);
 	for (i = 0; i < universe_count; i++) {
-#if 0
 		union {
 			const char *c;
 			char *s;
 		} foo;
-#endif
 		if (universes [i]) {
-			if (universes[i]->name_hash)
-			    option_name_free_hash_table(
-						&universes[i]->name_hash,
-						MDL);
-			if (universes[i]->code_hash)
-			    option_code_free_hash_table(
-						&universes[i]->code_hash,
-						MDL);
+			if (universes [i] -> hash)
+			    option_free_hash_table (&universes [i] -> hash,
+						    MDL);
 #if 0
 			if (universes [i] -> name > (char *)&end) {
 				foo.c = universes [i] -> name;
@@ -2700,9 +2859,7 @@ void free_everything(void)
 	relinquish_free_binding_values ();
 	relinquish_free_option_caches ();
 	relinquish_free_packets ();
-#if defined(COMPACT_LEASES)
 	relinquish_lease_hunks ();
-#endif
 	relinquish_hash_bucket_hunks ();
 	omapi_type_relinquish ();
 }
